@@ -1,4 +1,5 @@
-from typing import Any, Tuple
+from typing import Any, Tuple, Optional
+from uuid import uuid4
 
 from qottoauth.api.base import QottoAuthApi, QottoAuthApiError
 from qottoauth.models import (
@@ -43,10 +44,12 @@ class QottoAuthTestApi(QottoAuthApi):
         self._roles: dict[str, Role] = {}
         self._role_authorizations: dict[str, set[Authorization]] = {}
         self._members: dict[str, Member] = {}
+        self._member_cookies: dict[str, tuple[str, str]] = {}
         self._member_roles: dict[str, set[Role]] = {}
         self._member_authorizations: dict[str, set[Authorization]] = {}
         self._organizations: dict[str, Organization] = {}
         self._users: dict[str, User] = {}
+        self._user_cookies: dict[str, tuple[str, str]] = {}
         self._identities: dict[str, Identity] = {}
 
     def query(
@@ -154,14 +157,37 @@ class QottoAuthTestApi(QottoAuthApi):
             id=self._users[id].user_id,
             name=self._users[id].name,
             isSuperuser=self._users[id].is_superuser,
+            accounts=[
+                dict(
+                    id=user_account.account_id,
+                    application=self.application(user_account.application.application_id),
+                    user=dict(
+                        id=self._users[id].user_id,
+                        name=self._users[id].name,
+                        isSuperuser=self._users[id].is_superuser,
+                    ),
+                    enabled=user_account.enabled,
+                    data=user_account.data,
+
+                ) for user_account in [
+                    account for account in self._accounts.values() if account.user.user_id == id
+                ]
+            ],
         ) if id in self._users else None
 
+    def userFromCookies(self, tokenCookie, secretCookie):
+        for user_id, (user_token, user_secret) in self._user_cookies.items():
+            if user_token == tokenCookie and user_secret == secretCookie:
+                return self.user(user_id)
+        return None
+
     def users(self):
         return [self.user(id) for id in self._users]
 
     def createUser(self, name):
         user = User(gen_id(), name, False)
         self._users[user.user_id] = user
+        self._user_cookies[user.user_id] = (str(uuid4()), str(uuid4()))
         return dict(user=self.user(user.user_id))
 
     def createUserFromIdentity(self, identityId):
@@ -330,6 +356,12 @@ class QottoAuthTestApi(QottoAuthApi):
                             self._member_authorizations[memberId]],
         ) if memberId in self._members else None
 
+    def memberFromCookies(self, tokenCookie, secretCookie):
+        for member_id, (member_token, member_secret) in self._member_cookies.items():
+            if tokenCookie == member_token and secretCookie == member_secret:
+                return self.member(member_id)
+        return None
+
     def members(self):
         return [self.member(id) for id in self._members]
 
@@ -341,6 +373,7 @@ class QottoAuthTestApi(QottoAuthApi):
         self._members[member.member_id] = member
         self._member_roles[member.member_id] = set()
         self._member_authorizations[member.member_id] = set()
+        self._member_cookies[member.member_id] = (str(uuid4()), str(uuid4()))
         return dict(member=self.member(member.member_id))
 
     def updateMember(
@@ -363,6 +396,66 @@ class QottoAuthTestApi(QottoAuthApi):
             del self._members[memberId]
             del self._member_roles[memberId]
             del self._member_authorizations[memberId]
+            del self._member_cookies[memberId]
             return dict(deleted=True)
         else:
             return dict(deleted=False)
+
+    def cookies(self, userId=None, organizationId=None):
+        token, secret = None, None
+        if organizationId is None:
+            token, secret = self._user_cookies[userId]
+        else:
+            for member_id, (member_token, member_secret) in self._member_cookies.items():
+                if (
+                        self._members[member_id].user.user_id == userId
+                        and self._members[member_id].organization.organization_id == organizationId
+                ):
+                    token, secret = member_token, member_secret
+        if not token or not secret:
+            raise ValueError(f"No cookies found for user #{userId} in organization #{organizationId}.")
+        return [
+            dict(name='token', value=token, domain='domain', maxAge=3600, secure=True, httpOnly=False),
+            dict(name='secret', value=secret, domain='domain', maxAge=3600, secure=True, httpOnly=True),
+        ]
+
+    def isAuthorized(
+            self,
+            userId: str = None, memberId: str = None,
+            permissionId: str = None,
+            organizationId: str = None,
+            organizationNamespace: str = None,
+    ) -> bool:
+        user: Optional[User] = None
+        if userId:
+            user = self._users[userId]
+        member: Optional[Member] = None
+        if memberId:
+            member = self._members[memberId]
+        if not permissionId:
+            raise ValueError("No permissionId provided.")
+        permission: Permission = self._permissions[permissionId]
+        namespace: Optional[Namespace] = None
+        if organizationId:
+            namespace = self._organizations[organizationId].namespace
+        if organizationNamespace:
+            namespace = Namespace(organizationNamespace)
+        if not member:
+            return user.is_superuser if user else False
+        all_authorizations = set()
+        for role in self._member_roles[member.member_id]:
+            for authorization in self._role_authorizations[role.role_id]:
+                all_authorizations.add(authorization)
+        for authorization in self._member_authorizations[member.member_id]:
+            all_authorizations.add(authorization)
+        for authorization in all_authorizations:
+            if permission in self._authorization_permissions[authorization.authorization_id]:
+                if not namespace:
+                    return True
+                if not authorization.organization.namespace >= namespace:
+                    continue
+                if not authorization.inheritance and authorization.organization.namespace != namespace:
+                    continue
+                if namespace.matches(member.organization.namespace, authorization.matching):
+                    return True
+        return False
